@@ -33,8 +33,10 @@
 #include <sys/types.h>
 #include <sys/ioctl.h>
 #include <linux/sockios.h>
+#include <linux/types.h>
 #include <net/if.h>
 #include <pci/pci.h>
+#include <asm/types.h>
 #include <linux/ethtool.h>
 #include "efi.h"
 #include "efichar.h"
@@ -720,7 +722,7 @@ append_extra_args_file(void *data, unsigned long maxchars)
 
 	if (fd == -1) {
 		perror("Failed to open extra arguments file");
-		return 0;
+		exit(1);
 	}
 
 	do {
